@@ -22,18 +22,11 @@ try:
 except ImportError:  # pragma: NO COVER
     import mock
 
-from collections.abc import Iterable
+from collections.abc import AsyncIterable, Iterable
 import json
 import math
 
-from google.api_core import gapic_v1, grpc_helpers, grpc_helpers_async, path_template
-from google.api_core import api_core_version, client_options
-from google.api_core import exceptions as core_exceptions
-from google.api_core import retry as retries
-import google.auth
-from google.auth import credentials as ga_credentials
-from google.auth.exceptions import MutualTLSChannelError
-from google.oauth2 import service_account
+from google.api_core import api_core_version
 from google.protobuf import json_format
 import grpc
 from grpc.experimental import aio
@@ -43,6 +36,22 @@ import pytest
 from requests import PreparedRequest, Request, Response
 from requests.sessions import Session
 
+try:
+    from google.auth.aio import credentials as ga_credentials_async
+
+    HAS_GOOGLE_AUTH_AIO = True
+except ImportError:  # pragma: NO COVER
+    HAS_GOOGLE_AUTH_AIO = False
+
+from google.api_core import gapic_v1, grpc_helpers, grpc_helpers_async, path_template
+from google.api_core import client_options
+from google.api_core import exceptions as core_exceptions
+from google.api_core import retry as retries
+import google.auth
+from google.auth import credentials as ga_credentials
+from google.auth.exceptions import MutualTLSChannelError
+from google.oauth2 import service_account
+
 from google.cloud.binaryauthorization_v1.services.validation_helper_v1 import (
     ValidationHelperV1AsyncClient,
     ValidationHelperV1Client,
@@ -51,10 +60,24 @@ from google.cloud.binaryauthorization_v1.services.validation_helper_v1 import (
 from google.cloud.binaryauthorization_v1.types import service
 
 
+async def mock_async_gen(data, chunk_size=1):
+    for i in range(0, len(data)):  # pragma: NO COVER
+        chunk = data[i : i + chunk_size]
+        yield chunk.encode("utf-8")
+
+
 def client_cert_source_callback():
     return b"cert bytes", b"key bytes"
 
 
+# TODO: use async auth anon credentials by default once the minimum version of google-auth is upgraded.
+# See related issue: https://github.com/googleapis/gapic-generator-python/issues/2107.
+def async_anonymous_credentials():
+    if HAS_GOOGLE_AUTH_AIO:
+        return ga_credentials_async.AnonymousCredentials()
+    return ga_credentials.AnonymousCredentials()
+
+
 # If default endpoint is localhost, then default mtls endpoint will be the same.
 # This method modifies the default endpoint so the client can produce a different
 # mtls endpoint for endpoint testing purposes.
@@ -1200,27 +1223,6 @@ def test_validate_attestation_occurrence(request_type, transport: str = "grpc"):
     assert response.denial_reason == "denial_reason_value"
 
 
-def test_validate_attestation_occurrence_empty_call():
-    # This test is a coverage failsafe to make sure that totally empty calls,
-    # i.e. request == None and no flattened fields passed, work.
-    client = ValidationHelperV1Client(
-        credentials=ga_credentials.AnonymousCredentials(),
-        transport="grpc",
-    )
-
-    # Mock the actual call within the gRPC stub, and fake the request.
-    with mock.patch.object(
-        type(client.transport.validate_attestation_occurrence), "__call__"
-    ) as call:
-        call.return_value.name = (
-            "foo"  # operation_request.operation in compute client(s) expect a string.
-        )
-        client.validate_attestation_occurrence()
-        call.assert_called()
-        _, args, _ = call.mock_calls[0]
-        assert args[0] == service.ValidateAttestationOccurrenceRequest()
-
-
 def test_validate_attestation_occurrence_non_empty_request_with_auto_populated_field():
     # This test is a coverage failsafe to make sure that UUID4 fields are
     # automatically populated, according to AIP-4235, with non-empty requests.
@@ -1295,32 +1297,6 @@ def test_validate_attestation_occurrence_use_cached_wrapped_rpc():
         assert mock_rpc.call_count == 2
 
 
-@pytest.mark.asyncio
-async def test_validate_attestation_occurrence_empty_call_async():
-    # This test is a coverage failsafe to make sure that totally empty calls,
-    # i.e. request == None and no flattened fields passed, work.
-    client = ValidationHelperV1AsyncClient(
-        credentials=ga_credentials.AnonymousCredentials(),
-        transport="grpc_asyncio",
-    )
-
-    # Mock the actual call within the gRPC stub, and fake the request.
-    with mock.patch.object(
-        type(client.transport.validate_attestation_occurrence), "__call__"
-    ) as call:
-        # Designate an appropriate return value for the call.
-        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
-            service.ValidateAttestationOccurrenceResponse(
-                result=service.ValidateAttestationOccurrenceResponse.Result.VERIFIED,
-                denial_reason="denial_reason_value",
-            )
-        )
-        response = await client.validate_attestation_occurrence()
-        call.assert_called()
-        _, args, _ = call.mock_calls[0]
-        assert args[0] == service.ValidateAttestationOccurrenceRequest()
-
-
 @pytest.mark.asyncio
 async def test_validate_attestation_occurrence_async_use_cached_wrapped_rpc(
     transport: str = "grpc_asyncio",
@@ -1329,7 +1305,7 @@ async def test_validate_attestation_occurrence_async_use_cached_wrapped_rpc(
     # instead of constructing them on each call
     with mock.patch("google.api_core.gapic_v1.method_async.wrap_method") as wrapper_fn:
         client = ValidationHelperV1AsyncClient(
-            credentials=ga_credentials.AnonymousCredentials(),
+            credentials=async_anonymous_credentials(),
             transport=transport,
         )
 
@@ -1344,22 +1320,23 @@ async def test_validate_attestation_occurrence_async_use_cached_wrapped_rpc(
         )
 
         # Replace cached wrapped function with mock
-        mock_object = mock.AsyncMock()
+        mock_rpc = mock.AsyncMock()
+        mock_rpc.return_value = mock.Mock()
         client._client._transport._wrapped_methods[
             client._client._transport.validate_attestation_occurrence
-        ] = mock_object
+        ] = mock_rpc
 
         request = {}
         await client.validate_attestation_occurrence(request)
 
         # Establish that the underlying gRPC stub method was called.
-        assert mock_object.call_count == 1
+        assert mock_rpc.call_count == 1
 
         await client.validate_attestation_occurrence(request)
 
         # Establish that a new wrapper was not created for this call
         assert wrapper_fn.call_count == 0
-        assert mock_object.call_count == 2
+        assert mock_rpc.call_count == 2
 
 
 @pytest.mark.asyncio
@@ -1368,7 +1345,7 @@ async def test_validate_attestation_occurrence_async(
     request_type=service.ValidateAttestationOccurrenceRequest,
 ):
     client = ValidationHelperV1AsyncClient(
-        credentials=ga_credentials.AnonymousCredentials(),
+        credentials=async_anonymous_credentials(),
         transport=transport,
     )
 
@@ -1442,7 +1419,7 @@ def test_validate_attestation_occurrence_field_headers():
 @pytest.mark.asyncio
 async def test_validate_attestation_occurrence_field_headers_async():
     client = ValidationHelperV1AsyncClient(
-        credentials=ga_credentials.AnonymousCredentials(),
+        credentials=async_anonymous_credentials(),
     )
 
     # Any value that is part of the HTTP/1.1 URI should be sent as
@@ -1473,50 +1450,6 @@ async def test_validate_attestation_occurrence_field_headers_async():
     ) in kw["metadata"]
 
 
-@pytest.mark.parametrize(
-    "request_type",
-    [
-        service.ValidateAttestationOccurrenceRequest,
-        dict,
-    ],
-)
-def test_validate_attestation_occurrence_rest(request_type):
-    client = ValidationHelperV1Client(
-        credentials=ga_credentials.AnonymousCredentials(),
-        transport="rest",
-    )
-
-    # send a request that will satisfy transcoding
-    request_init = {"attestor": "projects/sample1/attestors/sample2"}
-    request = request_type(**request_init)
-
-    # Mock the http request call within the method and fake a response.
-    with mock.patch.object(type(client.transport._session), "request") as req:
-        # Designate an appropriate value for the returned response.
-        return_value = service.ValidateAttestationOccurrenceResponse(
-            result=service.ValidateAttestationOccurrenceResponse.Result.VERIFIED,
-            denial_reason="denial_reason_value",
-        )
-
-        # Wrap the value into a proper Response obj
-        response_value = Response()
-        response_value.status_code = 200
-        # Convert return value to protobuf type
-        return_value = service.ValidateAttestationOccurrenceResponse.pb(return_value)
-        json_return_value = json_format.MessageToJson(return_value)
-
-        response_value._content = json_return_value.encode("UTF-8")
-        req.return_value = response_value
-        response = client.validate_attestation_occurrence(request)
-
-    # Establish that the response is the type that we expect.
-    assert isinstance(response, service.ValidateAttestationOccurrenceResponse)
-    assert (
-        response.result == service.ValidateAttestationOccurrenceResponse.Result.VERIFIED
-    )
-    assert response.denial_reason == "denial_reason_value"
-
-
 def test_validate_attestation_occurrence_rest_use_cached_wrapped_rpc():
     # Clients should use _prep_wrapped_messages to create cached wrapped rpcs,
     # instead of constructing them on each call
@@ -1664,97 +1597,6 @@ def test_validate_attestation_occurrence_rest_unset_required_fields():
     )
 
 
-@pytest.mark.parametrize("null_interceptor", [True, False])
-def test_validate_attestation_occurrence_rest_interceptors(null_interceptor):
-    transport = transports.ValidationHelperV1RestTransport(
-        credentials=ga_credentials.AnonymousCredentials(),
-        interceptor=None
-        if null_interceptor
-        else transports.ValidationHelperV1RestInterceptor(),
-    )
-    client = ValidationHelperV1Client(transport=transport)
-    with mock.patch.object(
-        type(client.transport._session), "request"
-    ) as req, mock.patch.object(
-        path_template, "transcode"
-    ) as transcode, mock.patch.object(
-        transports.ValidationHelperV1RestInterceptor,
-        "post_validate_attestation_occurrence",
-    ) as post, mock.patch.object(
-        transports.ValidationHelperV1RestInterceptor,
-        "pre_validate_attestation_occurrence",
-    ) as pre:
-        pre.assert_not_called()
-        post.assert_not_called()
-        pb_message = service.ValidateAttestationOccurrenceRequest.pb(
-            service.ValidateAttestationOccurrenceRequest()
-        )
-        transcode.return_value = {
-            "method": "post",
-            "uri": "my_uri",
-            "body": pb_message,
-            "query_params": pb_message,
-        }
-
-        req.return_value = Response()
-        req.return_value.status_code = 200
-        req.return_value.request = PreparedRequest()
-        req.return_value._content = (
-            service.ValidateAttestationOccurrenceResponse.to_json(
-                service.ValidateAttestationOccurrenceResponse()
-            )
-        )
-
-        request = service.ValidateAttestationOccurrenceRequest()
-        metadata = [
-            ("key", "val"),
-            ("cephalopod", "squid"),
-        ]
-        pre.return_value = request, metadata
-        post.return_value = service.ValidateAttestationOccurrenceResponse()
-
-        client.validate_attestation_occurrence(
-            request,
-            metadata=[
-                ("key", "val"),
-                ("cephalopod", "squid"),
-            ],
-        )
-
-        pre.assert_called_once()
-        post.assert_called_once()
-
-
-def test_validate_attestation_occurrence_rest_bad_request(
-    transport: str = "rest", request_type=service.ValidateAttestationOccurrenceRequest
-):
-    client = ValidationHelperV1Client(
-        credentials=ga_credentials.AnonymousCredentials(),
-        transport=transport,
-    )
-
-    # send a request that will satisfy transcoding
-    request_init = {"attestor": "projects/sample1/attestors/sample2"}
-    request = request_type(**request_init)
-
-    # Mock the http request call within the method and fake a BadRequest error.
-    with mock.patch.object(Session, "request") as req, pytest.raises(
-        core_exceptions.BadRequest
-    ):
-        # Wrap the value into a proper Response obj
-        response_value = Response()
-        response_value.status_code = 400
-        response_value.request = Request()
-        req.return_value = response_value
-        client.validate_attestation_occurrence(request)
-
-
-def test_validate_attestation_occurrence_rest_error():
-    client = ValidationHelperV1Client(
-        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
-    )
-
-
 def test_credentials_transport_error():
     # It is an error to provide credentials and a transport instance.
     transport = transports.ValidationHelperV1GrpcTransport(
@@ -1847,18 +1689,248 @@ def test_transport_adc(transport_class):
         adc.assert_called_once()
 
 
+def test_transport_kind_grpc():
+    transport = ValidationHelperV1Client.get_transport_class("grpc")(
+        credentials=ga_credentials.AnonymousCredentials()
+    )
+    assert transport.kind == "grpc"
+
+
+def test_initialize_client_w_grpc():
+    client = ValidationHelperV1Client(
+        credentials=ga_credentials.AnonymousCredentials(), transport="grpc"
+    )
+    assert client is not None
+
+
+# This test is a coverage failsafe to make sure that totally empty calls,
+# i.e. request == None and no flattened fields passed, work.
+def test_validate_attestation_occurrence_empty_call_grpc():
+    client = ValidationHelperV1Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="grpc",
+    )
+
+    # Mock the actual call, and fake the request.
+    with mock.patch.object(
+        type(client.transport.validate_attestation_occurrence), "__call__"
+    ) as call:
+        call.return_value = service.ValidateAttestationOccurrenceResponse()
+        client.validate_attestation_occurrence(request=None)
+
+        # Establish that the underlying stub method was called.
+        call.assert_called()
+        _, args, _ = call.mock_calls[0]
+        request_msg = service.ValidateAttestationOccurrenceRequest()
+
+        assert args[0] == request_msg
+
+
+def test_transport_kind_grpc_asyncio():
+    transport = ValidationHelperV1AsyncClient.get_transport_class("grpc_asyncio")(
+        credentials=async_anonymous_credentials()
+    )
+    assert transport.kind == "grpc_asyncio"
+
+
+def test_initialize_client_w_grpc_asyncio():
+    client = ValidationHelperV1AsyncClient(
+        credentials=async_anonymous_credentials(), transport="grpc_asyncio"
+    )
+    assert client is not None
+
+
+# This test is a coverage failsafe to make sure that totally empty calls,
+# i.e. request == None and no flattened fields passed, work.
+@pytest.mark.asyncio
+async def test_validate_attestation_occurrence_empty_call_grpc_asyncio():
+    client = ValidationHelperV1AsyncClient(
+        credentials=async_anonymous_credentials(),
+        transport="grpc_asyncio",
+    )
+
+    # Mock the actual call, and fake the request.
+    with mock.patch.object(
+        type(client.transport.validate_attestation_occurrence), "__call__"
+    ) as call:
+        # Designate an appropriate return value for the call.
+        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
+            service.ValidateAttestationOccurrenceResponse(
+                result=service.ValidateAttestationOccurrenceResponse.Result.VERIFIED,
+                denial_reason="denial_reason_value",
+            )
+        )
+        await client.validate_attestation_occurrence(request=None)
+
+        # Establish that the underlying stub method was called.
+        call.assert_called()
+        _, args, _ = call.mock_calls[0]
+        request_msg = service.ValidateAttestationOccurrenceRequest()
+
+        assert args[0] == request_msg
+
+
+def test_transport_kind_rest():
+    transport = ValidationHelperV1Client.get_transport_class("rest")(
+        credentials=ga_credentials.AnonymousCredentials()
+    )
+    assert transport.kind == "rest"
+
+
+def test_validate_attestation_occurrence_rest_bad_request(
+    request_type=service.ValidateAttestationOccurrenceRequest,
+):
+    client = ValidationHelperV1Client(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+    # send a request that will satisfy transcoding
+    request_init = {"attestor": "projects/sample1/attestors/sample2"}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a BadRequest error.
+    with mock.patch.object(Session, "request") as req, pytest.raises(
+        core_exceptions.BadRequest
+    ):
+        # Wrap the value into a proper Response obj
+        response_value = mock.Mock()
+        json_return_value = ""
+        response_value.json = mock.Mock(return_value={})
+        response_value.status_code = 400
+        response_value.request = mock.Mock()
+        req.return_value = response_value
+        client.validate_attestation_occurrence(request)
+
+
 @pytest.mark.parametrize(
-    "transport_name",
+    "request_type",
     [
-        "grpc",
-        "rest",
+        service.ValidateAttestationOccurrenceRequest,
+        dict,
     ],
 )
-def test_transport_kind(transport_name):
-    transport = ValidationHelperV1Client.get_transport_class(transport_name)(
-        credentials=ga_credentials.AnonymousCredentials(),
+def test_validate_attestation_occurrence_rest_call_success(request_type):
+    client = ValidationHelperV1Client(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
     )
-    assert transport.kind == transport_name
+
+    # send a request that will satisfy transcoding
+    request_init = {"attestor": "projects/sample1/attestors/sample2"}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = service.ValidateAttestationOccurrenceResponse(
+            result=service.ValidateAttestationOccurrenceResponse.Result.VERIFIED,
+            denial_reason="denial_reason_value",
+        )
+
+        # Wrap the value into a proper Response obj
+        response_value = mock.Mock()
+        response_value.status_code = 200
+
+        # Convert return value to protobuf type
+        return_value = service.ValidateAttestationOccurrenceResponse.pb(return_value)
+        json_return_value = json_format.MessageToJson(return_value)
+        response_value.content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+        response = client.validate_attestation_occurrence(request)
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, service.ValidateAttestationOccurrenceResponse)
+    assert (
+        response.result == service.ValidateAttestationOccurrenceResponse.Result.VERIFIED
+    )
+    assert response.denial_reason == "denial_reason_value"
+
+
+@pytest.mark.parametrize("null_interceptor", [True, False])
+def test_validate_attestation_occurrence_rest_interceptors(null_interceptor):
+    transport = transports.ValidationHelperV1RestTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+        interceptor=None
+        if null_interceptor
+        else transports.ValidationHelperV1RestInterceptor(),
+    )
+    client = ValidationHelperV1Client(transport=transport)
+
+    with mock.patch.object(
+        type(client.transport._session), "request"
+    ) as req, mock.patch.object(
+        path_template, "transcode"
+    ) as transcode, mock.patch.object(
+        transports.ValidationHelperV1RestInterceptor,
+        "post_validate_attestation_occurrence",
+    ) as post, mock.patch.object(
+        transports.ValidationHelperV1RestInterceptor,
+        "pre_validate_attestation_occurrence",
+    ) as pre:
+        pre.assert_not_called()
+        post.assert_not_called()
+        pb_message = service.ValidateAttestationOccurrenceRequest.pb(
+            service.ValidateAttestationOccurrenceRequest()
+        )
+        transcode.return_value = {
+            "method": "post",
+            "uri": "my_uri",
+            "body": pb_message,
+            "query_params": pb_message,
+        }
+
+        req.return_value = mock.Mock()
+        req.return_value.status_code = 200
+        return_value = service.ValidateAttestationOccurrenceResponse.to_json(
+            service.ValidateAttestationOccurrenceResponse()
+        )
+        req.return_value.content = return_value
+
+        request = service.ValidateAttestationOccurrenceRequest()
+        metadata = [
+            ("key", "val"),
+            ("cephalopod", "squid"),
+        ]
+        pre.return_value = request, metadata
+        post.return_value = service.ValidateAttestationOccurrenceResponse()
+
+        client.validate_attestation_occurrence(
+            request,
+            metadata=[
+                ("key", "val"),
+                ("cephalopod", "squid"),
+            ],
+        )
+
+        pre.assert_called_once()
+        post.assert_called_once()
+
+
+def test_initialize_client_w_rest():
+    client = ValidationHelperV1Client(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+    assert client is not None
+
+
+# This test is a coverage failsafe to make sure that totally empty calls,
+# i.e. request == None and no flattened fields passed, work.
+def test_validate_attestation_occurrence_empty_call_rest():
+    client = ValidationHelperV1Client(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # Mock the actual call, and fake the request.
+    with mock.patch.object(
+        type(client.transport.validate_attestation_occurrence), "__call__"
+    ) as call:
+        client.validate_attestation_occurrence(request=None)
+
+        # Establish that the underlying stub method was called.
+        call.assert_called()
+        _, args, _ = call.mock_calls[0]
+        request_msg = service.ValidateAttestationOccurrenceRequest()
+
+        assert args[0] == request_msg
 
 
 def test_transport_grpc_default():
@@ -2410,36 +2482,41 @@ def test_client_with_default_client_info():
         prep.assert_called_once_with(client_info)
 
 
-@pytest.mark.asyncio
-async def test_transport_close_async():
-    client = ValidationHelperV1AsyncClient(
-        credentials=ga_credentials.AnonymousCredentials(),
-        transport="grpc_asyncio",
+def test_transport_close_grpc():
+    client = ValidationHelperV1Client(
+        credentials=ga_credentials.AnonymousCredentials(), transport="grpc"
     )
     with mock.patch.object(
-        type(getattr(client.transport, "grpc_channel")), "close"
+        type(getattr(client.transport, "_grpc_channel")), "close"
+    ) as close:
+        with client:
+            close.assert_not_called()
+        close.assert_called_once()
+
+
+@pytest.mark.asyncio
+async def test_transport_close_grpc_asyncio():
+    client = ValidationHelperV1AsyncClient(
+        credentials=async_anonymous_credentials(), transport="grpc_asyncio"
+    )
+    with mock.patch.object(
+        type(getattr(client.transport, "_grpc_channel")), "close"
     ) as close:
         async with client:
             close.assert_not_called()
         close.assert_called_once()
 
 
-def test_transport_close():
-    transports = {
-        "rest": "_session",
-        "grpc": "_grpc_channel",
-    }
-
-    for transport, close_name in transports.items():
-        client = ValidationHelperV1Client(
-            credentials=ga_credentials.AnonymousCredentials(), transport=transport
-        )
-        with mock.patch.object(
-            type(getattr(client.transport, close_name)), "close"
-        ) as close:
-            with client:
-                close.assert_not_called()
-            close.assert_called_once()
+def test_transport_close_rest():
+    client = ValidationHelperV1Client(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+    with mock.patch.object(
+        type(getattr(client.transport, "_session")), "close"
+    ) as close:
+        with client:
+            close.assert_not_called()
+        close.assert_called_once()
 
 
 def test_client_ctx():
